package ru.dbotthepony.mc.otm.triggers

import com.google.gson.JsonObject
import com.mojang.serialization.Codec
import com.mojang.serialization.codecs.RecordCodecBuilder
import net.minecraft.advancements.Criterion
import net.minecraft.advancements.critereon.AbstractCriterionTriggerInstance
import net.minecraft.advancements.critereon.ContextAwarePredicate
import net.minecraft.advancements.critereon.DeserializationContext
import net.minecraft.advancements.critereon.ItemPredicate
import net.minecraft.advancements.critereon.SimpleCriterionTrigger
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.data.Codec2TriggerSerializer
import java.util.Optional

class ItemTrigger(val id: ResourceLocation) : SimpleCriterionTrigger<ItemTrigger.Instance>() {
	fun trigger(player: ServerPlayer, item: ItemStack) {
		trigger(player) { if (it.invert) !it.predicate.matches(item) else it.predicate.matches(item) }
	}

	val codec = Codec2TriggerSerializer<Instance> { p ->
		RecordCodecBuilder.create {
			it.group(
				ItemPredicate.CODEC.fieldOf("predicate").forGetter(Instance::predicate),
				Codec.BOOL.optionalFieldOf("invert", false).forGetter(Instance::invert),
				p.playerPredicate.forGetter { it.playerPredicate() }
			).apply(it, ::Instance)
		}
	}

	override fun createInstance(p_66248_: JsonObject, p_286603_: Optional<ContextAwarePredicate>, p_66250_: DeserializationContext): Instance {
		return codec.fromJson(p_66248_, p_286603_, p_66250_)
	}

	inner class Instance(val predicate: ItemPredicate, val invert: Boolean = false, player: Optional<ContextAwarePredicate> = Optional.empty()) : AbstractCriterionTriggerInstance(player) {
		override fun serializeToJson(): JsonObject {
			return codec.toJson(this)
		}

		fun criterion() = Criterion(this@ItemTrigger, this)
	}
}
